package hamsam.protocol.secure;

import hamsam.api.Buddy;
import hamsam.api.Conference;
import hamsam.api.IMListener;
import hamsam.api.Message;
import hamsam.api.SmileyComponent;
import hamsam.exception.IllegalArgumentException;
import hamsam.exception.IllegalStateException;
import hamsam.exception.UnsupportedOperationException;
import hamsam.net.ProxyInfo;
import hamsam.protocol.Protocol;

public class SecureProtocol implements Protocol {

	private IMListener listener;

	//Don't worry about this method
	public void changeStatus(String status) throws IllegalArgumentException,
			IllegalStateException {
		System.out.println("Changing status");
	}

	//Implement this
	public void connect(String username, String password, String server,
			ProxyInfo info) throws IllegalStateException {
		System.out.println("Connecting");


	        //Call listener.connecting to indicate to the GUI that we are in the processing establishing a connection with the server.	
		listener.connecting(this);
		

		//You probably want to spawn a thread here.
		//The thread will try to connect to the server.  If you try to connect to the server without spawning a thread,
		//The GUI will freeze while we wait for the server to respond.
		//


		//After the connection has been established, tell the GUI that the connection is complete by calling listener.connected
		//However, hamsam demands that the buddy list be populated BEFORE listener.connected is called.
		//Note that the call to listener.buddyListReceived and the call to listener.connected do not actually belong
		//here in the SecureProtocol.connect method.  They should be located in your other thread.
		//They are placed here only to illustrate how to call them

		Buddy[] buddies = new Buddy[0];
		listener.buddyListReceived(this, buddies);


		listener.connected(this);

	}

	//Implement this
	public void disconnect() throws IllegalStateException {

		System.out.println("Disconnecting");
		listener.disconnected(this);
	}

	//Modify this
	public String getProtocolName() {
		//TODO change this string so that it includes your group's number.
		return "CS161 Fa06 Group #N's protocol";
	}

	//Ignore these	
	public boolean isBuddyAddRequestSupported() { 	return true;	}
	public boolean isBuddyNameAliasSupported() {	return false;	}
	public boolean isBuddyGroupSupported() {		return false;	}
	public boolean isIgnoreSupported() {			return true;	}
	public boolean isOfflineMessageSupported() {	return false;	}
	public boolean isTypingNotifySupported() {		return false;	}
	public boolean isConferenceSupported() {		return true;	}
	public boolean isMailNotifySupported() {		return false;	}
	public boolean isInvisibleSupported() {			return false;	}


	//This is probably ok the way it is.
	public void setListener(IMListener listener) {
		System.out.println("Setting Listener");
		this.listener = listener;
		
	
	}

	//Impelement this
	public void startConference(Conference conf, String message)
			throws UnsupportedOperationException, IllegalStateException {
		System.out.println("Starting Conference");


		//conferenceInvitationReceived should be called in your other thread when a request to join a conference comes across the network.
		//It is placed here to show you how to call it.
	        listener.conferenceInvitationReceived(conf, message);

	

	}

	//Implement this
	public void quitConference(Conference conf)
			throws UnsupportedOperationException, IllegalStateException {
			System.out.println("Quitting Conference");

	}

	//Implement this
	public void sendConferenceMessage(Conference conf, Message message)
			throws UnsupportedOperationException, IllegalStateException {
			System.out.println("Sending a conference message");

		
		        //conferenceParticipantJoined should actually be called in your other thread, 
			//and only when a message is received that the user has agree to join the conference.
		        listener.conferenceParticipantJoined(conf, new Buddy(this, "SomeBuddy"));

			//conferenceMessageReceived should be called by your other thread in response to an appropiate message
			//coming from the server
		        listener.conferenceMessageReceived(conf, new Buddy(this, "SomeBuddy"), message);


	}

	//Implement this
	public void addToBuddyList(Buddy buddy) throws IllegalArgumentException,  IllegalStateException {
		System.out.println("Adding to buddy list");
		listener.buddyAdded(buddy);

	}

	//Implement this
	public void deleteFromBuddyList(Buddy buddy)
			throws IllegalArgumentException, IllegalStateException {
			System.out.println("Deleting From Buddy List");
			listener.buddyDeleted(buddy);

	}

	//Don't worry about this
	public void ignoreBuddy(Buddy buddy) throws UnsupportedOperationException,
			IllegalStateException {
				System.out.println("Ignoring Buddy");

	}

	//Don't worry about this
	public void unignoreBuddy(Buddy buddy)
			throws UnsupportedOperationException, IllegalStateException {
			System.out.println("Unignoring Buddy");

	}

	//Implement this
	public void sendInstantMessage(Buddy buddy, Message message)
			throws IllegalStateException {
			System.out.println("Sending instant message");
			
		        //instantMessageReceived should actually be called in the other thread, and only when a message is in fact received
			//here we are merely showing how to call it.
			listener.instantMessageReceived(buddy, message);

	}

	//Don't worry about this
	public void changeBuddyAlias(Buddy buddy, String alias)
			throws UnsupportedOperationException {
			System.out.println("Changing Buddy Alias");
	}

	//Don't worry about this
	public void typingStarted(Buddy buddy)
			throws UnsupportedOperationException, IllegalStateException {
			
			//It is OK to leave this empty.
	}

	//Don't worry about this
	public void typingStopped(Buddy buddy)
			throws UnsupportedOperationException, IllegalStateException {
			
			//It is OK to leave this empty.

	}

	//Don't worry about this
	public SmileyComponent[] getSupportedSmileys() {
		//It is OK to leave this empty.
		return null;
	}


	//Don't worry about this
	public String[] getSupportedStatusMessages() {
		return null;
	}

	

}
